import logging


logger = logging.getLogger("coffee")

console_handler = logging.StreamHandler()
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
console_handler.setFormatter(formatter)
logger.addHandler(console_handler)



from os import getenv
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .api_client import CoffeeClient


class _Config:
    api_key: str = getenv("COFFEE_API_KEY")
    _client: "CoffeeClient" = None

    @property
    def api_client(self) -> "CoffeeClient":
        from .api_client import CoffeeClient

        if not self._client:
            self._client = CoffeeClient()
        return self._client

    @api_client.setter
    def api_client(self, cl: "CoffeeClient"):
        self._client = cl


config = _Config()



from typing import Optional

from pydantic import BaseModel, Field


class Model(BaseModel):
    id: str = Field(..., alias="model")
    name: Optional[str] = None
    provider: Optional[str] = Field(None, validate_default=True)


class OpenAI(Model):
    id: str = "gpt-4o"
    name: str = "OpenAIChat"
    provider: str = "OpenAI"


class DeepSeekChat(OpenAI):
    id: str = "deepseek-chat"
    name: str = "DeepSeekChat"
    provider: str = "DeepSeek"
    base_url: str = "https://api.deepseek.com"


class Claude(Model):
    id: str = "claude-3-5-sonnet-20241022"
    name: str = "Claude"
    provider: str = "Anthropic"


# MIGHT NOT WORK
class xAI(OpenAI):
    id: str = "grok-beta"
    name: str = "xAI"
    provider: str = "xAI"
    base_url: Optional[str] = "https://api.x.ai/v1"


class Groq(Model):
    id: str = "llama3-groq-70b-8192-tool-use-preview"
    name: str = "Groq"
    provider: str = "Groq"



from typing import Optional

from pydantic import BaseModel, Field


class UserOut(BaseModel):
    id_: str
    user_name: str
    name: str
    is_admin: bool
    avatar_url: Optional[str] = None


class CurrentUser(UserOut):
    secure: dict = Field(alias="_secure_")



from .agent import Agent
from .api_client import CoffeeClient
from .config import config
from .session import AgentSession

__all__ = ["config", Agent.__name__, CoffeeClient.__name__, AgentSession.__name__]



from pydantic import BaseModel
from .agent import Agent
from .log import logger
from rich.markdown import Markdown
from rich.panel import Panel
from rich.console import Console

console = Console()


class Task(BaseModel):
    description: str
    assignee: Agent

    def execute(self, message: str):
        prev = self.assignee.session_id
        self.assignee.create_session()
        res = self.assignee.message(
            f"<description>{self.description}</description>\n<message>{message}</message>",
            only_show_response=True,
        )
        self.assignee.session_id = prev
        return res


class Workflow(BaseModel):
    name: str = "Unnamed Workflow"
    steps: list[Task]
    _current_index: int = 0

    def add_task(self, task: Task):
        return Workflow(steps=[*self.steps, task])

    @staticmethod
    def build(task: Task):
        return Workflow(steps=[task])

    def execute(self, message: str):
        logger.debug(f"Executing: step={self._current_index}")
        if self._current_index == len(self.steps):
            return message

        res = self.steps[self._current_index].execute(message)
        logger.debug(f"step={self._current_index} Response: {res}"[:150])
        self._current_index += 1
        return self.execute(res)

    def execute_and_print(self, message: str):
        result = self.execute(message)
        user = Markdown(message)
        agent = Markdown(result)

        p = Panel(user, title="User")
        r = Panel(
            agent, title="Workflow:" + " > ".join((x.assignee.name for x in self.steps))
        )
        console.print(p)
        console.print(r)



import requests

from .config import config
from .types.user import CurrentUser
from .util.time_cache import time_cache
from .log import logger
from requests import Response
from functools import wraps


def response_handler(fn):
    @wraps(fn)
    def _wrapped(*args, **kwargs):
        resp: Response = fn(*args, **kwargs)
        js = text = None
        try:
            js = resp.json()
        except Exception as _:
            text = resp.text()
        try:
            resp.raise_for_status()
            if js and "error" in js:
                raise Exception("_")
        except Exception as _:
            msg = f"JSON={js}" if js else f"TEXT={text}"
            raise Exception(f"request({args=}, {kwargs=}) failed. \n{msg}")
        if js:
            match js:
                case list():
                    return js
                case dict({"data": x}):
                    return x
                case default:
                    return default
        return resp

    return _wrapped


class CoffeeClient:
    def __init__(
        self, base_url: str = "https://coffee-sdk-api.fly.dev", api_key: str = None
    ):
        self.base_url = base_url.rstrip("/")
        config.api_key = api_key or config.api_key
        if not config.api_key:
            raise ValueError(
                "API key must be provided or set in environment variable API_KEY"
            )
        self.session = requests.Session()
        self.session.headers.update({"x-api-key": config.api_key})

    @property
    @time_cache(max_age=300)
    def user_details(self) -> CurrentUser:
        logger.debug("Getting user details")
        gm = self.get_me()["user_data"]
        logger.debug(f"get_me() => {gm}")
        return CurrentUser.model_validate(gm)

    # Agents routes
    @response_handler
    def create_agent(self, agent_data: dict) -> requests.Response:
        url = f"{self.base_url}/agent/-/create?overwrite"
        return self.session.post(url, json=agent_data)

    @response_handler
    def add_agent_knowledge_url(self, agent_id: str, url: str) -> requests.Response:
        url_path = f"{self.base_url}/agent/{agent_id}/add-knowledge"
        return self.session.post(url_path, json={"url": url})

    @response_handler
    def get_all_agents(self) -> requests.Response:
        url = f"{self.base_url}/agent/-/getall"
        return self.session.get(url)

    @response_handler
    def create_agent_session(self, agent_id: str) -> requests.Response:
        url = f"{self.base_url}/agent/{agent_id}/session/create"
        return self.session.get(url)

    @response_handler
    def get_agent_sessions(self, agent_id: str) -> requests.Response:
        url = f"{self.base_url}/agent/{agent_id}/sessions"
        return self.session.get(url)

    @response_handler
    def run_session(
        self, session_id: str, message: str, only_show_response: bool = False
    ) -> str:
        url = f"{self.base_url}/agent/{session_id}/run"
        return self.session.post(
            url,
            json={"message": message},
            params={"only_show_response": ""} if only_show_response else None,
        )

    @response_handler
    def create_team_session(self) -> requests.Response:
        url = f"{self.base_url}/agent/team/session/create"
        return self.session.get(url)

    @response_handler
    def get_team_session_history(self, session_id: str) -> requests.Response:
        url = f"{self.base_url}/agent/team/{session_id}/history"
        return self.session.get(url)

    @response_handler
    def run_team_session(self, session_id: str, message: str) -> requests.Response:
        url = f"{self.base_url}/agent/team/{session_id}/run"
        return self.session.post(url, json={"message": message})

    @response_handler
    def get_public_agents(self) -> requests.Response:
        url = f"{self.base_url}/agent/public/agents"
        return self.session.get(url)

    @response_handler
    def get_agent(self, agent_id: str) -> requests.Response:
        url = f"{self.base_url}/agent/get/{agent_id}"
        return self.session.get(url)

    @response_handler
    def get_session_details(self, session_id: str) -> requests.Response:
        url = f"{self.base_url}/agent/{session_id}/details"
        return self.session.get(url)

    @response_handler
    def upload_knowledge_file(self, agent_id: str, file_path: str) -> requests.Response:
        url = f"{self.base_url}/knowledge/{agent_id}/upload-knowledge"
        with open(file_path, "rb") as f:
            files = {"file": f}
            return self.session.post(url, files=files)

    @response_handler
    def create_plugin(self, plugin_data: dict) -> requests.Response:
        url = f"{self.base_url}/plugins/-/create"
        return self.session.post(url, json=plugin_data)

    @response_handler
    def enable_plugin_for_agent(
        self, plugin_id: str, agent_id: str
    ) -> requests.Response:
        url = f"{self.base_url}/plugins/{plugin_id}/enable-for/{agent_id}"
        return self.session.get(url)

    @response_handler
    def get_user_plugins(self) -> requests.Response:
        url = f"{self.base_url}/plugins/user-plugins"
        return self.session.get(url)

    @response_handler
    def enable_builtin_plugin(
        self, plugin_name: str, agent_id: str
    ) -> requests.Response:
        url = f"{self.base_url}/plugins/builtins/{plugin_name}/enable-for/{agent_id}"
        return self.session.post(url)

    @response_handler
    def enable_marketplace_plugin(
        self, plugin_id: str, agent_id: str
    ) -> requests.Response:
        url = f"{self.base_url}/plugins/marketplace/plugins/{plugin_id}/enable-for/{agent_id}"
        return self.session.post(url)

    @response_handler
    def get_marketplace_plugins(self) -> requests.Response:
        url = f"{self.base_url}/plugins/marketplace/plugins/all"
        return self.session.get(url)

    @response_handler
    def get_marketplace_plugin(self, plugin_id: str) -> requests.Response:
        url = f"{self.base_url}/plugins/marketplace/{plugin_id}/plugin"
        return self.session.get(url)

    @response_handler
    def buy_credits(
        self, amount: int, coffee_credits: int, tx_hash: str, network: str, wallet: str
    ) -> requests.Response:
        url = f"{self.base_url}/transaction/buy-credits"
        data = {
            "amount": amount,
            "coffee_credits": coffee_credits,
            "hash": tx_hash,
            "network": network,
            "wallet": wallet,
        }
        return self.session.post(url, json=data)

    @response_handler
    def buy_agent(self, agent_id: str, amount: int, tx_hash: str) -> requests.Response:
        url = f"{self.base_url}/transaction/{agent_id}/buy-agent"
        data = {"amount": amount, "hash": tx_hash}
        return self.session.post(url, json=data)

    @response_handler
    def buy_plugin(
        self, plugin_id: str, amount: int, tx_hash: str
    ) -> requests.Response:
        url = f"{self.base_url}/transaction/{plugin_id}/buy-plugin"
        data = {"amount": amount, "hash": tx_hash}
        return self.session.post(url, json=data)

    @response_handler
    def upload_user_avatar(self, data: bytes) -> requests.Response:
        url = f"{self.base_url}/uploads/user-avatar"
        return self.session.post(url, data=data)

    def upload_file(self, file_path: str, data: bytes) -> requests.Response:
        url = f"{self.base_url}/uploads/{file_path}"
        return self.session.post(url, data=data)

    @response_handler
    def github_auth(self, code: str) -> requests.Response:
        url = f"{self.base_url}/users/auth/github"
        return self.session.post(url, json={"code": code})

    @response_handler
    def get_me(self) -> dict:
        url = f"{self.base_url}/users/me"
        logger.debug(f"{url=}")
        return self.session.get(url)

    @response_handler
    def get_user_sessions_data(self, user_id: str) -> requests.Response:
        url = f"{self.base_url}/users/{user_id}/sessions-data"
        return self.session.get(url)



from typing import Annotated, Literal, Optional

from pydantic import BaseModel, HttpUrl, StringConstraints

from ..config import config

from ..log import logger


class PluginArgType(BaseModel):
    type: Literal["string", "number", "integer", "object", "array", "boolean", "null"]
    description: Optional[str] = None


class PluginIn(BaseModel):
    name: Annotated[
        str,
        StringConstraints(
            strip_whitespace=True,
            min_length=1,
            max_length=64,
            pattern="^[a-zA-Z0-9_-]+$",
        ),
    ]
    description: Annotated[
        str,
        StringConstraints(
            min_length=1,
            max_length=1024,
        ),
    ]
    about: Optional[str]
    url: Optional[HttpUrl] = None
    args: dict[str, PluginArgType]


class RemoteTool(BaseModel):
    name: str
    url: str
    args: dict[str, PluginArgType]
    instructions: str
    tool_id: str

    _is_registered: bool = False

    def register_tool(self):
        if self._is_registered:
            return
        logger.debug(f"Registering <Tool={self.name=}, {self.url=}")
        config.api_client.create_plugin(self.model_dump(mode="json"))
        self._is_registered = True



from .config import config
from .types.agent import AgentIn as _AgentIn
from .util.hash import generate_deterministic_hash
from rich.console import Console
from rich.markdown import Markdown
from rich.panel import Panel
from .log import logger

console = Console()


class Agent(_AgentIn):
    team: list["Agent"] = []
    _is_registered: bool = False

    def _register_if_not_exists(self):
        if self._is_registered:
            return
        if not self.agent_id:
            self.agent_id = generate_deterministic_hash(
                f"{config.api_client.user_details.id_}/{self.name}",
                "sha256",
            )
            logger.debug(f"Generated following agent id: {self.agent_id}")

        for k in self.team:
            logger.debug("Registering teams that might need to be registered")
            k._register_if_not_exists()

        for tool in self.tools:
            if isinstance(tool, str):
                continue
            tool.register_tool()

        logger.debug(f"Registering agent {self.name}")
        logger.debug(
            f"Register result: {config.api_client.create_agent(self.model_dump(mode='json'))}",
        )
        self._is_registered = True

    def create_session(self):
        self._register_if_not_exists()
        sess = config.api_client.create_agent_session(self.agent_id)
        self.session_id = sess["id_"]

    def message(self, msg: str, only_show_response: bool = True):
        self._register_if_not_exists()
        self.create_session()
        return config.api_client.run_session(self.session_id, msg, only_show_response)

    def print_message(self, msg: str):
        resp = self.message(msg, True)

        user = Markdown(msg)
        agent = Markdown(resp)

        p = Panel(user, title="User")
        r = Panel(agent, title=self.name)
        console.print(p)
        console.print(r)



from typing import Annotated, Any, Optional, Union, Literal

from pydantic import BaseModel, StringConstraints

from .model import Model, OpenAI
from .tool import RemoteTool

DEFAULT_MODEL = OpenAI(id="gpt-4o")
StringNoSpacesNoSpecialChars = Annotated[
    str,
    StringConstraints(
        strip_whitespace=True,
        min_length=1,
        max_length=64,
        pattern="^[a-zA-Z0-9_-]+$",
    ),
]
BuiltinTools = Literal[
    "web3_essentials_tools",
    "solana_tools",
    "eth_tools",
    "calculator",
    "crawl4ai_tools",
    "csv_tools",
    "dalle",
    "duckdb_tools",
    "duckduckgo",
    "elevenlabs_tools",
    "googlesearch",
    "hackers_news",
    "jina_reader_tools",
    "newspaper_tools",
    "pubmed",
    "replicate_toolkit",
    "spider",
    "web_browser",
    "wikipedia_tools",
    "yfinance_tools",
    "youtube_tools",
    "giphy_tools",
    "sleep",
]


class AgentIn(BaseModel):
    name: StringNoSpacesNoSpecialChars
    guidelines: Optional[list[str]] = None
    instructions: Optional[Union[list[str], str]] = None
    short_description: Optional[str] = None
    tools: list[Union[BuiltinTools, RemoteTool]] = []
    model: Model = DEFAULT_MODEL
    team: list[Any] = []
    imageUrl: str = "https://tivzaxmpxiyucjvvdtko.supabase.co/storage/v1/object/public/coffee-pre-prod-2025/default.png"
    session_id: Optional[str] = None
    agent_id: Optional[str] = None
